from typing import Dict, Set, Optional

from transmart_loader.transmart import TreeNode, ConceptNode, Concept, ValueType


class OntologyMapper:
    """
    Map ontology nodes to TranSMART
    """
    def __init__(self, system: str):
        self.system = system
        self.root_nodes: Set[TreeNode] = set()
        self.nodes: Dict[str, TreeNode] = {}
        self.concepts: Set[Concept] = set()

    def map_code(self, parent_code: Optional[str], code: str, label: str) -> None:
        concept_code = self.system + '/' + code
        concept = Concept(concept_code, label, concept_code, ValueType.Categorical)
        self.concepts.add(concept)
        node = ConceptNode(concept)
        if parent_code is not None and parent_code in self.nodes:
            parent_node = self.nodes[parent_code]
            parent_node.add_child(node)
        else:
            self.root_nodes.add(node)
        self.nodes[code] = node



import sys

import click
from python_claml import claml
from python_claml.claml_types import ClaML

from .ontology_writer import OntologyWriter

from transmart_loader.console import Console
from transmart_loader.copy_writer import TransmartCopyWriter
from transmart_loader.loader_exception import LoaderException


def run(system: str, input_file: str, output_dir: str, code_prefix: bool = False):
    """
    Reads a ClaML classification document and translates it to ontology terms
    in TranSMART.

    :param system: a unique identifier to identify the ontology
    :param input_file: the input ClaML document
    :param output_dir: the output directory to write the transmart-copy files to
    :param code_prefix: Whether to use the concept code as prefix for the concept label
    :return:
    """
    Console.title('ClaML to TranSMART')
    try:
        Console.info('Writing files to {}'.format(output_dir))
        copy_writer = TransmartCopyWriter(output_dir)
        ontology_writer = OntologyWriter(system)
        with open(input_file, 'r') as reader:
            Console.info('Reading file contents from {} ...'.format(input_file))
            contents = reader.read()
            Console.info('Parsing ClaML document ...')
            classification: ClaML = claml.CreateFromDocument(contents)
            for identifier in classification.Identifier:
                Console.info('Classification identifier: {}/{}'.format(identifier.authority, identifier.uid))
            for cls in classification.Class:
                ontology_writer.process_class(cls, code_prefix)
        ontology_writer.write(copy_writer)
        Console.info('Done.')
    except LoaderException as e:
        Console.error(e)
        sys.exit(1)


@click.command()
@click.argument('system')
@click.argument('input_file')
@click.argument('output_dir')
@click.option('--code-prefix', is_flag=True, show_default=True,
              help='Use the concept code as prefix for the concept label')
@click.version_option()
def claml2transmart(system: str, input_file: str, output_dir: str, code_prefix: bool):
    run(system, input_file, output_dir, code_prefix)


def main():
    claml2transmart()


if __name__ == '__main__':
    main()



from xml.dom import Node

from python_claml.claml_types import Class

from .ontology_mapper import OntologyMapper
from transmart_loader.copy_writer import TransmartCopyWriter


def normalise_code(code: str) -> str:
    return code.replace('!', '').replace('*', '')


def to_string(nodes):
    """
    Serialise a forest of DOM nodes to text,
    using the data fields of text nodes.
    :param nodes: the forest of DOM nodes
    :return: a concatenation of string representations.
    """
    result = []
    for node in nodes:
        if node.nodeType == Node.TEXT_NODE:
            result.append(node.data)
        else:
            result.append(to_string(node.childNodes))
    return ''.join(result)


class OntologyWriter:

    def process_class(self, cls: Class, code_prefix: bool = False) -> None:
        class_label = None
        for rubric in cls.Rubric:
            if rubric.kind == 'preferred':
                for label in rubric.Label:
                    class_label = to_string(label.toDOM().childNodes)
        class_label = cls.code + '. ' + class_label if code_prefix else class_label
        if len(cls.SuperClass) > 0:
            for superClass in cls.SuperClass:
                self.mapper.map_code(superClass.code, cls.code, class_label)
        else:
            self.mapper.map_code(None, cls.code, class_label)

    def write(self, writer: TransmartCopyWriter):
        for concept in self.mapper.concepts:
            writer.visit_concept(concept)
        for node in self.mapper.root_nodes:
            writer.visit_node(node)

    def __init__(self, system: str):
        self.mapper = OntologyMapper(system)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
from setuptools import setup

if sys.version_info < (3, 6):
    sys.exit(
        'Python < 3.6 is not supported. You are using Python {}.{}.'.format(
            sys.version_info[0], sys.version_info[1])
    )

here = os.path.abspath(os.path.dirname(__file__))

# To update the package version number, edit claml2transmart/__version__.py
version = {}
with open(os.path.join(here, 'claml2transmart', '__version__.py')) as f:
    exec(f.read(), version)

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('requirements.txt', 'r') as f:
    required_packages = f.read().splitlines()

setup(
    name='claml2transmart',
    version=version['__version__'],
    description="Example ClaML to TranSMART loader",
    long_description=readme + '\n\n',
    author="Gijs Kant",
    author_email='gijs@thehyve.nl',
    url='https://github.com/thehyve/python_claml2transmart',
    packages=[
        'claml2transmart',
    ],
    package_dir={'claml2transmart':
                 'claml2transmart'},
    entry_points={
        'console_scripts': ['claml2transmart=claml2transmart.claml2transmart:main'],
    },
    include_package_data=True,
    license="MIT",
    zip_safe=False,
    keywords='claml2transmart',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    python_requires='>=3.6.0',
    install_requires=required_packages,
    setup_requires=[
        'pygments',
        # dependency for `python setup.py test`
        'pytest-runner',
        # dependencies for `python setup.py build_sphinx`
        'sphinx',
        'sphinx_rtd_theme',
        'recommonmark',
        # dependency for `python setup.py bdist_wheel`
        'wheel'
    ],
    tests_require=[
        'pytest',
        'pytest-cov',
        'pycodestyle',
    ],
    extras_require={
        'dev':  ['prospector[with_pyroma]', 'yapf', 'isort'],
    }
)
